@@ -44,11 +44,11 @@
 #' @export
 get_anchors <- function(relation) {
 
-  # this allows the data to be accessed without attaching the package
-  alist <- eval(parse(text = "text2map::anchor_lists"))
-  anchor_list <- alist[alist$relation == relation, c("add", "subtract")]
+    # this allows the data to be accessed without attaching the package
+    alist <- eval(parse(text = "text2map::anchor_lists"))
+    anchor_list <- alist[alist$relation == relation, c("add", "subtract")]
 
-  return(anchor_list)
+    return(anchor_list)
 }
 
 #' Word embedding semantic direction extractor
@@ -128,15 +128,15 @@ get_anchors <- function(relation) {
 #'
 #' # create anchor list
 #' gen <- data.frame(
-#'   add = c("woman"),
-#'   subtract = c("man")
+#'     add = c("woman"),
+#'     subtract = c("man")
 #' )
 #'
 #' dir <- get_direction(anchors = gen, wv = ft_wv_sample)
 #'
 #' dir <- get_direction(
-#'   anchors = gen, wv = ft_wv_sample,
-#'   method = "PCA", n = 1L
+#'     anchors = gen, wv = ft_wv_sample,
+#'     method = "PCA", n = 1L
 #' )
 #' @export
 get_direction <- function(anchors, wv,
@@ -144,91 +144,91 @@ get_direction <- function(anchors, wv,
                           missing = "stop",
                           n_dirs = 1L) {
 
-  # check that word vectors exist for each word
-  anchors <- .check_term_in_embeddings(anchors, wv, action = missing)
+    # check that word vectors exist for each word
+    anchors <- .check_term_in_embeddings(anchors, wv, action = missing)
 
-  if (!is.null(ncol(anchors)) &&
-    ncol(anchors) == 2) {
-    anchors_add <- anchors[, 1, drop = TRUE]
-    anchors_sub <- anchors[, 2, drop = TRUE]
-  } else {
-    if (length(anchors) == 2) {
-      anchors_add <- anchors[[1]]
-      anchors_sub <- anchors[[2]]
+    if (!is.null(ncol(anchors)) &&
+        ncol(anchors) == 2) {
+        anchors_add <- anchors[, 1, drop = TRUE]
+        anchors_sub <- anchors[, 2, drop = TRUE]
     } else {
-      stop("get_direction requires two sets of juxtaposing terms")
+        if (length(anchors) == 2) {
+            anchors_add <- anchors[[1]]
+            anchors_sub <- anchors[[2]]
+        } else {
+            stop("get_direction requires two sets of juxtaposing terms")
+        }
     }
-  }
 
-  # PAIRED
-  # take the mean of a set of word vector differences
-  # between a collection of antonym word pairs as used
-  # in Kozlowski et al. 2019 and Stoltz and Taylor 2019
-  if (method == "paired") {
+    # PAIRED
+    # take the mean of a set of word vector differences
+    # between a collection of antonym word pairs as used
+    # in Kozlowski et al. 2019 and Stoltz and Taylor 2019
+    if (method == "paired") {
 
-    # subtract vectors for words in column 2 from words in column 1
-    direction <- wv[anchors_add, , drop = FALSE] -
-      wv[anchors_sub, , drop = FALSE]
-    # get the average of the resulting differences
-    direction <- t(as.matrix(colMeans(direction)))
-  }
+        # subtract vectors for words in column 2 from words in column 1
+        direction <- wv[anchors_add, , drop = FALSE] -
+            wv[anchors_sub, , drop = FALSE]
+        # get the average of the resulting differences
+        direction <- t(as.matrix(colMeans(direction)))
+    }
 
-  # POOLED
-  # average  the vectors for words on each pole,
-  # then take the difference between these two average
-  # as used in Larsen et al 2015 and Arseniev-Koehler and Foster 2020
-  if (method == "pooled") {
-    mu1 <- t(as.matrix(colMeans(
-      wv[anchors_add, , drop = FALSE]
-    )))
-    mu2 <- t(as.matrix(colMeans(
-      wv[anchors_sub, , drop = FALSE]
-    )))
-    direction <- mu1 - mu2
-  }
+    # POOLED
+    # average  the vectors for words on each pole,
+    # then take the difference between these two average
+    # as used in Larsen et al 2015 and Arseniev-Koehler and Foster 2020
+    if (method == "pooled") {
+        mu1 <- t(as.matrix(colMeans(
+            wv[anchors_add, , drop = FALSE]
+        )))
+        mu2 <- t(as.matrix(colMeans(
+            wv[anchors_sub, , drop = FALSE]
+        )))
+        direction <- mu1 - mu2
+    }
 
-  # L^2 Euclidean norm of PAIRED
-  # as used in Bolukbasi et al. 2016
-  # Quantifying and Reducing Stereotypes in Word Embeddings
-  if (method == "L2") {
-    direction <- wv[anchors_add, , drop = FALSE] -
-      wv[anchors_sub, , drop = FALSE]
-    # get the average of the resulting differences
-    direction <- t(as.matrix(colMeans(direction)))
-    # divide by Euclidean norm
-    direction <- direction / norm(direction, type = "2")
-  }
+    # L^2 Euclidean norm of PAIRED
+    # as used in Bolukbasi et al. 2016
+    # Quantifying and Reducing Stereotypes in Word Embeddings
+    if (method == "L2") {
+        direction <- wv[anchors_add, , drop = FALSE] -
+            wv[anchors_sub, , drop = FALSE]
+        # get the average of the resulting differences
+        direction <- t(as.matrix(colMeans(direction)))
+        # divide by Euclidean norm
+        direction <- direction / norm(direction, type = "2")
+    }
 
-  # Direction is the first principal component
-  # as used in Bolukbasi et al. 2016
-  # Man is to Computer Programmer as Woman is to Homemaker?
-  if (method == "PCA") {
-    # yes, these are inverted for a reason
-    direction <- wv[anchors_sub, , drop = FALSE] -
-      wv[anchors_add, , drop = FALSE]
-    direction <- Matrix::t(
-      stats::prcomp(direction,
-        center = TRUE,
-        scale. = nrow(direction) > 1L,
-        rank. = n_dirs
-      )$rotation
+    # Direction is the first principal component
+    # as used in Bolukbasi et al. 2016
+    # Man is to Computer Programmer as Woman is to Homemaker?
+    if (method == "PCA") {
+        # yes, these are inverted for a reason
+        direction <- wv[anchors_sub, , drop = FALSE] -
+            wv[anchors_add, , drop = FALSE]
+        direction <- Matrix::t(
+            stats::prcomp(direction,
+                center = TRUE,
+                scale. = nrow(direction) > 1L,
+                rank. = n_dirs
+            )$rotation
+        )
+    }
+
+    if (!method %in% c("pooled", "paired", "L2", "PCA")) {
+        stop("method must be 'pooled', 'paired', 'L2' or 'PCA'")
+    }
+
+    # create unique name for new vector
+    rownames(direction) <- make.unique(
+        replicate(
+            n_dirs,
+            paste0(anchors_add[1], "_pole")
+        ),
+        sep = "_"
     )
-  }
 
-  if (!method %in% c("pooled", "paired", "L2", "PCA")) {
-    stop("method must be 'pooled', 'paired', 'L2' or 'PCA'")
-  }
-
-  # create unique name for new vector
-  rownames(direction) <- make.unique(
-    replicate(
-      n_dirs,
-      paste0(anchors_add[1], "_pole")
-    ),
-    sep = "_"
-  )
-
-  return(direction)
+    return(direction)
 }
 
 #' Word embedding semantic centroid extractor
@@ -272,49 +272,49 @@ get_direction <- function(anchors, wv,
 #' @export
 get_centroid <- function(anchors, wv, missing = "stop") {
 
-  # if data.frame or matrix convert first column to list of terms
-  if (!is.null(ncol(anchors))) {
-    anchors <- unlist(anchors[, 1],
-      recursive = FALSE,
-      use.names = FALSE
-    )
-  } else {
-    if (is.recursive(anchors)) {
-      # if nested list, only first list is used
-      anchors <- unlist(anchors[[1]],
-        recursive = FALSE,
-        use.names = FALSE
-      )
+    # if data.frame or matrix convert first column to list of terms
+    if (!is.null(ncol(anchors))) {
+        anchors <- unlist(anchors[, 1],
+            recursive = FALSE,
+            use.names = FALSE
+        )
     } else {
-      anchors <- unlist(anchors,
+        if (is.recursive(anchors)) {
+            # if nested list, only first list is used
+            anchors <- unlist(anchors[[1]],
+                recursive = FALSE,
+                use.names = FALSE
+            )
+        } else {
+            anchors <- unlist(anchors,
+                recursive = FALSE,
+                use.names = FALSE
+            )
+        }
+    }
+
+    anchors <- unlist(
+        stringi::stri_split(
+            anchors,
+            fixed = " ",
+            omit_empty = TRUE
+        ),
         recursive = FALSE,
         use.names = FALSE
-      )
-    }
-  }
+    )
 
-  anchors <- unlist(
-    stringi::stri_split(
-      anchors,
-      fixed = " ",
-      omit_empty = TRUE
-    ),
-    recursive = FALSE,
-    use.names = FALSE
-  )
+    # check that word vectors exist for each word
+    anchors <- .check_term_in_embeddings(anchors, wv, action = missing)
 
-  # check that word vectors exist for each word
-  anchors <- .check_term_in_embeddings(anchors, wv, action = missing)
+    # select vectors for anchor words
+    centroid <- wv[anchors[(anchors %in% rownames(wv))], , drop = FALSE]
+    # average the resulting vector
+    centroid <- t(as.matrix(colMeans(centroid)))
 
-  # select vectors for anchor words
-  centroid <- wv[anchors[(anchors %in% rownames(wv))], , drop = FALSE]
-  # average the resulting vector
-  centroid <- t(as.matrix(colMeans(centroid)))
-
-  # create unique name for new vector
-  first_word <- strsplit(anchors[1], " ")[[1]][1]
-  rownames(centroid) <- paste0(first_word, "_centroid")
-  return(centroid)
+    # create unique name for new vector
+    first_word <- strsplit(anchors[1], " ")[[1]][1]
+    rownames(centroid) <- paste0(first_word, "_centroid")
+    return(centroid)
 }
 
 #' Word embedding semantic region extractor
@@ -383,10 +383,10 @@ get_centroid <- function(anchors, wv, missing = "stop") {
 #' data(ft_wv_sample)
 #'
 #' my.regions <- get_regions(
-#'   wv = ft_wv_sample,
-#'   k_regions = 10L,
-#'   max_iter = 10L,
-#'   seed = 01984
+#'     wv = ft_wv_sample,
+#'     k_regions = 10L,
+#'     max_iter = 10L,
+#'     seed = 01984
 #' )
 #' @export
 #'
@@ -394,22 +394,21 @@ get_regions <- function(wv,
                         k_regions = 5L,
                         max_iter = 20L,
                         seed = 0) {
+    regions <- ClusterR::KMeans_arma(
+        data = wv,
+        clusters = k_regions,
+        n_iter = max_iter,
+        seed = seed,
+        verbose = FALSE
+    )
 
-  regions <- ClusterR::KMeans_arma(
-    data = wv,
-    clusters = k_regions,
-    n_iter = max_iter,
-    seed = seed,
-    verbose = FALSE
-  )
+    regions <- as(regions, "matrix")
 
-  regions <- as(regions, "matrix")
+    rownames(regions) <- paste0("region_", seq_len(nrow(regions)))
 
-  rownames(regions) <- paste0("region_", seq_len(nrow(regions)))
+    regions <- Matrix::Matrix(regions, sparse = TRUE)
 
-  regions <- Matrix::Matrix(regions, sparse = TRUE)
-
-  return(regions)
+    return(regions)
 }
 
 #' Find the 'projection matrix' to a semantic vector
@@ -434,23 +433,23 @@ get_regions <- function(wv,
 #'
 #' @export
 find_projection <- function(wv, vec) {
-  stopifnot(ncol(wv) == length(vec))
-  # this would provide the projection of one vector
-  # to another vector (as opposed to a matrix)
-  # https://stackoverflow.com/a/62495884/15855390
-  # as.vector( (u %*% v) / (v %*% v) ) * v
+    stopifnot(ncol(wv) == length(vec))
 
-  vec <- as.vector(vec)
-  # RESULT will be orthogonal to vec
-  # adapted from `wordVectors` package
-  # t(x) %*% y is crossprod()
-  RESULT <- crossprod(
-    t(wv %*% vec) /
-      as.vector(vec %*% vec),
-    vec
-  )
+    # provides the projection of one vector
+    # to another vector (as opposed to a matrix)
+    # https://stackoverflow.com/a/62495884/15855390
+    # as.vector( (u %*% v) / (v %*% v) ) * v
+    vec <- as.vector(vec)
+    # RESULT will be orthogonal to vec
+    # adapted from `wordVectors` package
+    # t(x) %*% y is crossprod()
+    RESULT <- crossprod(
+        t(wv %*% vec) /
+            as.vector(vec %*% vec),
+        vec
+    )
 
-  return(RESULT)
+    return(RESULT)
 }
 
 #' Find the 'rejection matrix' from a semantic vector
@@ -470,12 +469,12 @@ find_projection <- function(wv, vec) {
 #'
 #' @export
 find_rejection <- function(wv, vec) {
-  stopifnot(ncol(wv) == length(vec))
-  # The projection of the matrix that
-  # is not parallel to the vector
-  # adapted from `wordVectors` package
-  RESULT <- wv - find_projection(wv, vec)
-  return(RESULT)
+    stopifnot(ncol(wv) == length(vec))
+    # The projection of the matrix that
+    # is not parallel to the vector
+    # adapted from `wordVectors` package
+    RESULT <- wv - find_projection(wv, vec)
+    return(RESULT)
 }
 
 
@@ -490,7 +489,9 @@ find_rejection <- function(wv, vec) {
 #' word embedding representations. The function also finds a transformed
 #' matrix that approximately aligns \eqn{B}, with another matrix,
 #' \eqn{A}, of word embedding vectors (reference), using Procrustes
-#' transformation (see details).
+#' transformation (see details). Finally, given a term-co-occurrence matrix
+#' built on a local corpus, the function can "retrofit" pretrained
+#' embeddings to better match the local corpus.
 #'
 #' @details
 #' Aligning a source matrix of word embedding vectors, \eqn{B}, to a
@@ -537,13 +538,15 @@ find_rejection <- function(wv, vec) {
 #' Borg and Groenen. (1997). \emph{Modern Multidimensional Scaling}.
 #' New York: Springer. 340-342
 #'
+#' @importFrom rsvd rsvd
+#'
 #' @param wv Matrix of word embedding vectors (a.k.a embedding model)
 #'           with rows as terms (the source matrix to be transformed).
 #' @param ref If `method = "align"`, this is the reference matrix
 #'            toward which the source matrix is to be aligned.
 #' @param method Character vector indicating the method to use for
 #'               the transformation. Current methods include: "align",
-#'               "norm", and "center" -- see details.
+#'               "norm", "center", and "refrofit" -- see details.
 #'
 #'
 #' @return A new word embedding matrix,
@@ -552,60 +555,89 @@ find_rejection <- function(wv, vec) {
 #' @export
 find_transformation <- function(wv,
                                 ref = NULL,
-                                method = c("align", "norm", "center")) {
-  stopifnot(inherits(wv, "matrix") || inherits(wv, "sparseMatrix"))
-  method <- match.arg(method)
+                                method = c(
+                                    "align", "norm",
+                                    "center", "retrofit"
+                                )) {
+    stopifnot(inherits(wv, "matrix") || inherits(wv, "sparseMatrix"))
+    method <- match.arg(method)
 
-  RESULT <- switch(method,
-    align = .procustes_align(wv, ref),
-    norm = .l2_normalize(wv),
-    center = .colmeans_translate(wv)
-  )
+    RESULT <- switch(method,
+        align = .procustes_align(wv, ref),
+        norm = .l2_normalize(wv),
+        center = .colmeans_translate(wv),
+        retrofit = .retrofit(wv, ref)
+    )
 
-  return(RESULT)
+    return(RESULT)
 }
 
 
 ## INTERNAL FUNCTIONS ## -------------------------------------------------------
 
 .procustes_align <- function(wv, ref) {
-  stopifnot(inherits(ref, "matrix") || inherits(ref, "sparseMatrix"))
-  stopifnot(ncol(wv) == ncol(ref))
+    stopifnot(ncol(wv) == ncol(ref))
 
-  # scaling (scale to unit length)
-  wv <- .l2_normalize(wv)
-  ref <- .l2_normalize(ref)
-  # column center (translation)
-  # following Schlechtweg et al. (2019)
-  wv <- .colmeans_translate(wv)
-  ref <- .colmeans_translate(ref)
-  # svd (rotation/reflection)
-  rot <- svd(crossprod(ref, wv))
-  Q <- tcrossprod(rot$v, rot$u) # Q = VU^T
+    # scaling (scale to unit length)
+    wv <- .l2_normalize(wv)
+    ref <- .l2_normalize(ref)
+    # column center (translation)
+    # following Schlechtweg et al. (2019)
+    wv <- .colmeans_translate(wv)
+    ref <- .colmeans_translate(ref)
+    # svd (rotation/reflection)
+    rot <- rsvd::rsvd(Matrix::crossprod(ref, wv))
+    Q <- Matrix::tcrossprod(rot$v, rot$u) # Q = VU^T
 
-  return(wv %*% Q)
+    return(wv %*% Q)
 }
 
 .colmeans_translate <- function(wv) {
-  return(wv - rep(
-    colMeans(wv),
-    rep.int(
-      nrow(wv),
-      ncol(wv)
-    )
-  ))
+    return(wv - rep(
+        Matrix::colMeans(wv),
+        rep.int(
+            nrow(wv),
+            ncol(wv)
+        )
+    ))
 }
 
-.l2_normalize <- function(wv) {
-  norm_vec <- 1 / sqrt(rowSums(wv^2))
-  # when sum row elements == 0
-  norm_vec[is.infinite(norm_vec)] <- 0
+.l2_normalize <- function(x) {
+    norm_vec <- 1 / sqrt(Matrix::rowSums(x^2))
+    # when sum row elements == 0
+    norm_vec[is.infinite(norm_vec)] <- 0
+    norm_vec[is.na(norm_vec)] <- 0
 
-  if (inherits(wv, "sparseMatrix")) {
-    return(Matrix::Diagonal(x = norm_vec) %*% wv)
-  } else {
-    return(wv * norm_vec)
-  }
+    # This is the old way...
+    # if (inherits(wv, "sparseMatrix")) {
+    #     return(Matrix::Diagonal(x = norm_vec) %*% wv)
+    # } else {
+    #     return(wv * norm_vec)
+    # }
+    return(x * norm_vec)
+}
+
+.retrofit <- function(wv, ref) {
+
+    # only keep shared vocab (this could be tweaked later)
+    vocab <- intersect(rownames(wv), rownames(ref))
+    wv <- wv[vocab, ]
+    ref <- ref[vocab, vocab]
+
+    # get SVD embeddings for TCM
+    ref <- find_transformation(ref, method = "norm")
+    ref <- rsvd::rsvd(ref, k = ncol(wv))
+    ref <- ref$v %*% diag(ref$d)
+
+    # align ref embeddings to pretrained
+    wv <- find_transformation(wv, method = "norm")
+    ref <- find_transformation(ref, wv, method = "align")
+
+    # add ref embeddings to pretrained
+    ref <- wv + ref
+    ref <- find_transformation(ref, method = "norm")
+
+    return(ref)
 }
 
 
@@ -616,11 +648,12 @@ find_transformation <- function(wv,
 #' makes sure all terms are present in the word embeddings provided.
 #'
 #' @details
-#'
 #' If `action = "remove"`, output is the same, but missing terms
 #' or rows with missing terms are removed. If `action = "stop"`, the function
 #' will stop with an error. In both cases, the missing terms are printed.
 #'
+#' @importFrom kit funique
+#'
 #' @param terms List or data.frame of terms or term pairs
 #' @param wv Matrix of word embedding vectors (a.k.a embedding model)
 #'           with rows as terms.
@@ -635,295 +668,105 @@ find_transformation <- function(wv,
 #'         missing terms or rows with missing terms removed
 #' @noRd
 .check_term_in_embeddings <- function(terms, wv, action = "stop") {
-  vocab <- unique(unlist(strsplit(unlist(terms), " ")))
-  bad_words <- unlist(vocab)[!(unlist(vocab) %in% rownames(wv))]
+    vocab <- kit::funique(unlist(strsplit(unlist(terms), " ")))
+    bad_words <- unlist(vocab)[!(unlist(vocab) %in% rownames(wv))]
 
-  ## If no missing terms, return terms ##
-  if (identical(bad_words, character(0))) {
-    return(terms)
-  }
-
-
-
-  ## Create the stop message ##
-  if (length(bad_words) > 10L) {
-    n_words <- length(bad_words) - 10L
-    bad_words_msg <- paste(bad_words[seq_len(10)], collapse = "; ")
-    bad_words_msg <- paste(bad_words_msg, " and ", n_words, " more term(s)")
-  } else {
-    bad_words_msg <- bad_words
-  }
-
-  stop_msg <- paste0(
-    "The following have no matching word vectors: ",
-    paste(bad_words, collapse = "; ")
-  )
-
-  if (action == "stop") {
-    stop(stop_msg)
-  }
-
-  ### If we want to remove the missing terms ###
-  if (action == "remove") {
-
-    # for DATA FRAMES or TIBBLES
-    if (is.data.frame(terms)) {
-      any_words <- paste(bad_words, collapse = "|")
-
-      if (ncol(terms) == 2) {
-        terms <- terms[!grepl(any_words, unlist(terms[, 1])), ]
-        terms <- terms[!grepl(any_words, unlist(terms[, 2])), ]
-        # stop if removing term creates an empty list
-        if (nrow(terms) == 0) {
-          stop(stop_msg)
-        }
-      }
-
-      if (ncol(terms) == 1) {
-        terms <- terms[!grepl(any_words, unlist(terms[, 1])), , drop = FALSE]
-        # stop if removing term creates an empty list
-        if (identical(terms, character(0)) ||
-          nrow(terms) == 0) {
-          stop(stop_msg)
-        }
-      }
+    ## If no missing terms, return terms ##
+    if (identical(bad_words, character(0))) {
+        return(terms)
     }
 
-    # if LIST
-    # lists are used for pooling rather than paired
-    if (!is.data.frame(terms) && is.list(terms)) {
-      if (length(terms) == 2) {
-        terms <- lapply(
-          terms,
-          function(x) {
-            x[!x %in% bad_words]
-          }
-        )
-
-        # stop if removing term creates an empty list
-        if (identical(terms[[1]], character(0))) {
-          stop(stop_msg)
-        }
-        if (identical(terms[[2]], character(0))) {
-          stop(stop_msg)
-        }
-      }
-
-      if (length(terms) == 1) {
-        terms <- list(terms[[1]][!terms[[1]] %in% bad_words])
-        # stop if removing term creates an empty list
-        if (identical(terms[[1]], character(0))) {
-          stop(stop_msg)
-        }
-      }
-    }
-
-    # if VECTOR
-    if (is.vector(terms)) {
-      terms <- terms[!terms %in% bad_words]
-      # stop if removing term creates an vector
-      if (length(unlist(terms)) == 0) {
-        stop(stop_msg)
-      }
-    }
-  }
-
-
-  message(
-    "The following (and any associated terms) removed
-because there are no matching word vectors: ",
-    paste0(paste(bad_words, collapse = "; "))
-  )
-
-  return(terms)
-}
-
-
-#' Read word embedding models from `bin` or `vec` files
-#'
-#' This an internal function, currently in development. Word embeddings are
-#' commonly in `bin` or `vec` (i.e. word2vec) format. This function will
-#' read these files into the `R` environment.
-#'
-#' @details
-#' Setting `n_vocab` will limit the number of rows that will be read in.
-#' As rows of embeddings tend to be sorted by their frequencies in the training
-#' corpus, this argument will return the most frequent terms in descending
-#' order. Reading may take a while and cannot be easily parallelized since
-#' connection objects cannot be "shared" across processes. Therefore,
-#' setting `n_vocab` can speed up this process.
-#'
-#' This function is adapted from \url{github.com/bmschmidt/wordVectors}
-#'
-#' @importFrom utils txtProgressBar setTxtProgressBar read.table
-#'
-#' @param path Path to word embedding model file
-#' @param dims Integer indicating number of dimensions of the embeddings. If
-#'             `NULL` (default), dimensions will be assumed from the file.
-#' @param format Character indicating whether it is a `bin` or `vec`,
-#'               If `NULL` (the default), the function will assume by the
-#'               ending of the file name.
-#' @param n_vocab Integer indicating number of rows (i.e. terms) to read in
-#'                to the environment (default is `Inf` which will read all).
-#'                See details.
-#'
-#'
-#' @return An embedding matrix
-#'
-#' @noRd
-.read_embeddings <- function(path,
-                             dims = NULL,
-                             format = NULL,
-                             n_vocab = Inf) {
-  if (is.null(format)) {
-    if (rev(strsplit(path, "\\.")[[1]])[1] == "bin") {
-      message("Assuming binary format...")
-      format <- "bin"
-    }
-
-    if (rev(strsplit(path, "\\.")[[1]])[1] == "vec") {
-      message("Assuming word2vec format...")
-      format <- "vec"
-    }
-  }
-
-
-  if (format == "bin") {
-
-    # open connection to file
-    con <- file(path, "rb")
-
-    # rows
-    row_number <- ""
-    mostRecent <- ""
-    while (mostRecent != " ") {
-      mostRecent <- readChar(con, 1)
-      row_number <- paste0(row_number, mostRecent)
-    }
-    row_number <- as.integer(row_number)
-
-    # cols
-    col_number <- ""
-    while (mostRecent != "\n") {
-      mostRecent <- readChar(con, 1)
-      col_number <- paste0(col_number, mostRecent)
-    }
-    col_number <- as.integer(col_number)
-
-
-    if (n_vocab < row_number) {
-      message(paste(
-        "Reading the first", n_vocab,
-        "rows of a binary file of",
-        row_number, "rows and", col_number, "columns"
-      ))
-      row_number <- n_vocab
+    ## Create the stop message ##
+    if (length(bad_words) > 10L) {
+        n_words <- length(bad_words) - 10L
+        bad_words_msg <- paste(bad_words[seq_len(10)], collapse = "; ")
+        bad_words_msg <- paste(bad_words_msg, " and ", n_words, " more term(s)")
     } else {
-      message(paste(
-        "Reading a binary file of", row_number,
-        "rows and", col_number, "columns"
-      ))
+        bad_words_msg <- bad_words
     }
 
-    returned_columns <- as.integer(col_number)
-    if (!is.null(dims)) {
-      returned_columns <- dims
-    }
-
-    ## Read a row
-    rownames <- rep("", row_number)
-
-    read_row_func <- function(x) {
-      # create progress bar
-      pb <- utils::txtProgressBar(
-        min = 0,
-        max = row_number,
-        style = 3
-      )
-
-      out <- vapply(
-        seq_len(x),
-        function(i, pb) {
-          utils::setTxtProgressBar(pb, i)
-          rowname <- ""
-          mostRecent <- ""
-          while (TRUE) {
-            mostRecent <- readChar(con, 1)
-            if (mostRecent == " ") {
-              break
-            }
-            if (mostRecent != "\n") {
-              # Some versions end with newlines, some don't.
-              rowname <- paste0(rowname, mostRecent)
-            }
-          }
-          rownames[i] <<- rowname
-
-          row <- readBin(con,
-            numeric(),
-            size = 4,
-            n = col_number,
-            endian = "little"
-          )
-
-          if (!is.null(dims)) {
-            row <- row[dims]
-          }
-
-          return(row)
-        },
-        FUN.VALUE = as.array(rep(0, returned_columns)),
-        pb = pb
-      )
-
-      close(pb)
-      close(con)
-
-      out <- t(out)
-      rownames(out) <- rownames
-
-      return(out)
-    }
-
-    RESULT <- read_row_func(row_number)
-
-    suppressWarnings(closeAllConnections())
-  }
-
-  if (format == "vec") {
-    if (is.null(dims)) {
-      test <- utils::read.table(
-        path,
-        header = FALSE,
-        skip = 1L,
-        nrows = 1L,
-        quote = "",
-        comment.char = ""
-      )
-
-      returned_columns <- ncol(test) - 1L
-    } else {
-      returned_columns <- dims
-    }
-
-    # read .vec files
-    wv_mat <- utils::read.table(
-      path,
-      header = FALSE,
-      skip = 1,
-      nrows = n_vocab,
-      colClasses = c("character", rep("numeric", returned_columns)),
-      quote = "",
-      comment.char = ""
+    stop_msg <- paste0(
+        "The following have no matching word vectors: ",
+        paste(bad_words, collapse = "; ")
     )
 
-    names(wv_mat)[1] <- "word"
-    wv_mat$word[is.na(wv_mat$word)] <- "NA"
+    if (action == "stop") {
+        stop(stop_msg)
+    }
 
-    RESULT <- as.matrix(wv_mat[, colnames(wv_mat) != "word"])
-    rownames(RESULT) <- wv_mat$word
-  }
+    ### If we want to remove the missing terms ###
+    if (action == "remove") {
 
-  colnames(RESULT) <- paste0("V", seq_len(returned_columns))
-  return(RESULT)
+        # for DATA FRAMES or TIBBLES
+        if (is.data.frame(terms)) {
+            any_words <- paste(bad_words, collapse = "|")
+
+            if (ncol(terms) == 2) {
+                terms <- terms[!grepl(any_words, unlist(terms[, 1])), ]
+                terms <- terms[!grepl(any_words, unlist(terms[, 2])), ]
+                # stop if removing term creates an empty list
+                if (nrow(terms) == 0) {
+                    stop(stop_msg)
+                }
+            }
+
+            if (ncol(terms) == 1) {
+                terms <- terms[!grepl(any_words, unlist(terms[, 1])), ,
+                    drop = FALSE
+                ]
+                # stop if removing term creates an empty list
+                if (identical(terms, character(0)) ||
+                    nrow(terms) == 0) {
+                    stop(stop_msg)
+                }
+            }
+        }
+
+        # if LIST
+        # lists are used for pooling rather than paired
+        if (!is.data.frame(terms) && is.list(terms)) {
+            if (length(terms) == 2) {
+                terms <- lapply(
+                    terms,
+                    function(x) {
+                        x[!x %in% bad_words]
+                    }
+                )
+
+                # stop if removing term creates an empty list
+                if (identical(terms[[1]], character(0))) {
+                    stop(stop_msg)
+                }
+                if (identical(terms[[2]], character(0))) {
+                    stop(stop_msg)
+                }
+            }
+
+            if (length(terms) == 1) {
+                terms <- list(terms[[1]][!terms[[1]] %in% bad_words])
+                # stop if removing term creates an empty list
+                if (identical(terms[[1]], character(0))) {
+                    stop(stop_msg)
+                }
+            }
+        }
+
+        # if VECTOR
+        if (is.vector(terms)) {
+            terms <- terms[!terms %in% bad_words]
+            # stop if removing term creates an vector
+            if (length(unlist(terms)) == 0) {
+                stop(stop_msg)
+            }
+        }
+    }
+
+
+    message(
+        "The following (and any associated terms) removed
+because there are no matching word vectors: ",
+        paste0(paste(bad_words, collapse = "; "))
+    )
+
+    return(terms)
 }
